# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import functools
import os
import os.path
import six
import sys
import time
from typing import Dict

from dotenv import load_dotenv, find_dotenv

from azure_devtools.scenario_tests.utilities import trim_kwargs_from_test_function

from . import mgmt_settings_fake as fake_settings
from .azure_testcase import (
    _is_autorest_v3,
    get_resource_name,
    get_qualified_method_name,
)
from .fake_credentials_async import AsyncFakeCredential
from .helpers import is_live
from .sanitizers import add_general_string_sanitizer


load_dotenv(find_dotenv())


def _sanitize_token(token, fake_token):
    add_general_string_sanitizer(value=fake_token, target=token)
    url_safe_token = token.replace("/", "%2F")
    add_general_string_sanitizer(value=fake_token, target=url_safe_token)
    async_token = token.replace("%3A", ":")
    add_general_string_sanitizer(value=fake_token, target=async_token)


class AzureRecordedTestCase(object):
    """Test class for use by data-plane tests that use the azure-sdk-tools test proxy.

    For more details and usage examples, refer to
    https://github.com/Azure/azure-sdk-for-python/blob/main/doc/dev/test_proxy_migration_guide.md
    """

    @property
    def settings(self):
        if self.is_live:
            return os.environ
        else:
            return fake_settings

    def _load_settings(self):
        return fake_settings, os.environ

    @property
    def is_live(self):
        return is_live()

    @property
    def qualified_test_name(self):
        return get_qualified_method_name(self, "method_name")

    @property
    def in_recording(self):
        return self.is_live

    # TODO: This needs to be removed, recording processors are handled on the proxy side, but
    # this is needed for the preparers
    @property
    def recording_processors(self):
        return []

    def is_playback(self):
        return not self.is_live

    def get_settings_value(self, key):
        key_value = os.environ.get("AZURE_" + key, None)

        if not key_value or self.is_playback():
            try:
                key_value = getattr(self.settings, key)
            except Exception as ex:
                six.raise_from(ValueError("Could not get {}".format(key)), ex)
        return key_value

    def get_credential(self, client_class, **kwargs):
        tenant_id = os.environ.get("AZURE_TENANT_ID", getattr(os.environ, "TENANT_ID", None))
        client_id = os.environ.get("AZURE_CLIENT_ID", getattr(os.environ, "CLIENT_ID", None))
        secret = os.environ.get("AZURE_CLIENT_SECRET", getattr(os.environ, "CLIENT_SECRET", None))
        is_async = kwargs.pop("is_async", False)

        if tenant_id and client_id and secret and self.is_live:
            if _is_autorest_v3(client_class):
                # Create azure-identity class
                from azure.identity import ClientSecretCredential

                if is_async:
                    from azure.identity.aio import ClientSecretCredential
                return ClientSecretCredential(tenant_id=tenant_id, client_id=client_id, client_secret=secret)
            else:
                # Create msrestazure class
                from msrestazure.azure_active_directory import (
                    ServicePrincipalCredentials,
                )

                return ServicePrincipalCredentials(tenant=tenant_id, client_id=client_id, secret=secret)
        else:
            if _is_autorest_v3(client_class):
                if is_async:
                    if self.is_live:
                        raise ValueError(
                            "Async live doesn't support mgmt_setting_real, please set AZURE_TENANT_ID, "
                            "AZURE_CLIENT_ID, AZURE_CLIENT_SECRET"
                        )
                    return AsyncFakeCredential()
                else:
                    return self.settings.get_azure_core_credentials()
            else:
                return self.settings.get_credentials()

    def create_client_from_credential(self, client_class, credential, **kwargs):

        # Real client creation
        # TODO decide what is the final argument for that
        # if self.is_playback():
        #     kwargs.setdefault("polling_interval", 0)
        if _is_autorest_v3(client_class):
            kwargs.setdefault("logging_enable", True)
            client = client_class(credential=credential, **kwargs)
        else:
            client = client_class(credentials=credential, **kwargs)

        if self.is_playback():
            try:
                client._config.polling_interval = 0  # FIXME in azure-mgmt-core, make this a kwargs
            except AttributeError:
                pass

        if hasattr(client, "config"):  # Autorest v2
            if self.is_playback():
                client.config.long_running_operation_timeout = 0
            client.config.enable_http_logger = True
        return client

    def create_random_name(self, name):
        unique_test_name = os.getenv("PYTEST_CURRENT_TEST").encode("utf-8")
        return get_resource_name(name, unique_test_name)

    def get_resource_name(self, name):
        """Alias to create_random_name for back compatibility."""
        return self.create_random_name(name)

    def get_replayable_random_resource_name(self, name):
        """In a replay scenario (not live), gives the static moniker. In the random scenario, gives generated name."""
        if self.is_live:
            created_name = self.create_random_name(name)
            self.scrubber.register_name_pair(created_name, name)
        return name

    def get_preparer_resource_name(self, prefix):
        """Random name generation for use by preparers.

        If prefix is a blank string, use the fully qualified test name instead.
        This is what legacy tests do for resource groups."""
        return self.get_resource_name(prefix)

    @staticmethod
    def await_prepared_test(test_fn):
        """Synchronous wrapper for async test methods. Used to avoid making changes
        upstream to AbstractPreparer, which only awaits async tests that use preparers.
        (Add @AzureTestCase.await_prepared_test decorator to async tests without preparers)

        # Note: this will only be needed so long as we maintain unittest.TestCase in our
        test-class inheritance chain.
        """

        if sys.version_info < (3, 5):
            raise ImportError("Async wrapper is not needed for Python 2.7 code.")

        import asyncio

        @functools.wraps(test_fn)
        def run(test_class_instance, *args, **kwargs):
            trim_kwargs_from_test_function(test_fn, kwargs)
            loop = asyncio.get_event_loop()
            return loop.run_until_complete(test_fn(test_class_instance, **kwargs))

        return run

    def sleep(self, seconds):
        if self.is_live:
            time.sleep(seconds)

    def generate_sas(self, *args, **kwargs):
        """Generates a SAS token using a generation function and arguments, and sanitizes token params in recordings.

        By default, this sanitizes the values of the `sig`, `st`, and `se` parameters to "fake_token_value", "start",
        and "end", respectively. By providing a dictionary of `fake_parameters` mapping parameter names to values they
        should be sanitized with, you can customize this sanitization and include other parameters.

        :keyword fake_parameters: A dictionary with token parameter names as keys, and the values to sanitize these keys
            with as values. For example: {"sktid": "00000000-0000-0000-0000-000000000000", "sig": "sanitized"}
        :paramtype fake_parameters: Dict[str, str]
        :keyword str fake_value: The value used to sanitize `sig`. Defaults to "fake_token_value".
        """
        sas_func = args[0]
        sas_func_pos_args = args[1:]

        fake_value = kwargs.pop("fake_value", "fake_token_value")
        fake_parameters = kwargs.pop("fake_parameters", {})
        token = sas_func(*sas_func_pos_args, **kwargs)

        fake_token = self._create_fake_token(token, fake_value, fake_parameters)
        _sanitize_token(token, fake_token)

        if self.is_live:
            return token
        return fake_token

    def _create_fake_token(self, token: str, fake_sig_value: str, fake_parameters: Dict[str, str]) -> str:
        """Returns a replacement value for sanitizing `sig`, `st`, `se`, and provided params of a given SAS token."""
        parameters = token.split("&")

        for idx, parameter in enumerate(parameters):
            key = parameter.split("=")[0]
            fake_value = fake_parameters.get(key)
            if fake_value is not None:
                parameters[idx] = "=".join([key, fake_value])
            elif key == "sig":
                parameters[idx] = "=".join([key, fake_sig_value])
            elif key == "st":
                parameters[idx] = "=".join([key, "start"])
            elif key == "se":
                parameters[idx] = "=".join([key, "end"])

        return "&".join(parameters)



from dataclasses import dataclass
from dwai.zhipuai.model_api.params import ModelParams

@dataclass
class ChatGLM130bParams(ModelParams):
    model: str = "chatglm_130b"


@dataclass
class ChatGLM6bParams(ModelParams):
    model: str = "chatglm_6b"



import json
import threading
from typing import Any, List, Dict
from abc import ABC, abstractmethod
import requests
from dwai.core.exceptions import CompletionsRequestError

_thread_context = threading.local()

class BaseCompletions(ABC):
    """ 对话API基类 """
    api_key = ""
    api_base = ""

    def __init__(self, api_key: str, api_base: str):
        self.api_key = api_key
        self.api_base = api_base

    @classmethod
    def call(cls, *args, **kwargs) -> Any:
        raise NotImplementedError()

    @abstractmethod
    def run(self, prompt: str, history: List[Dict[str, Any]], stream: bool, **kwargs: Any):
        pass

    def do_http(self, api_path, headers, data, stream):

        headers["Content-Type"] = "application/json;charset=UTF-8"
        headers["Authorization"] = "Bearer %s" % self.api_key
        headers["User-Agent "] = "dw"

        url = "%s%s" % (self.api_base, api_path)

        session = self.__get_session()
        resp = session.request("POST",
                               url=url,
                               headers=headers,
                               data=json.dumps(data),
                               stream=stream)

        if not resp.ok:
            raise CompletionsRequestError("completion request error", resp.status_code, resp.text)

        return resp

    @staticmethod
    def __get_session() -> requests.Session:
        if not hasattr(_thread_context, "session"):
            _thread_context.session = requests.Session()

        return _thread_context.session








class CompletionsRequestError(Exception):
    def __init__(self,
                 message=None,
                 status_code=None,
                 response_body=None
                 ):
        super(CompletionsRequestError, self).__init__(message)

        self.message = message
        self.status_code = status_code
        self.response_body = response_body

    def __str__(self):
        return "%s: status_code=%s, response_body=%s" % (
            self.message,
            self.status_code,
            self.response_body
        )

    def __repr__(self):
        return "(message=%r, status_code=%r, response_body=%r)" % (
            self.message,
            self.status_code,
            self.response_body
        )




import json
import dwai
from jsonpath_ng import parse
from dwai.azure.model_api.completions import AzureCompletions
from dwai.bailian.model_api.completions import Completions
from dwai.mini.model_api.completions import MiniCompletions
from dwai.pangu.model_api.completions import PanGuCompletions
from dwai.tione.v20211111.tione_client import TioneClient
from dwai.zhipuai.model_api.api import ModelAPI


def azure_qa(**kwargs):
    chat = AzureCompletions()
    json_output = chat.call(prompt=kwargs.get('prompt'), stream=False)
    matches = parse('$.choices[0].message.content').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def mini_qa(**kwargs):
    chat = MiniCompletions()
    json_output = chat.call(prompt=kwargs.get('prompt'), stream=False)
    matches = parse('$.choices[0].messages[0].text').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def dwai_bailian_qa(**kwargs):
    chat = Completions()
    json_output = chat.call(app_id="1e4ddc3659324ad0b8a0039230f1dba3", prompt=kwargs.get('prompt'))
    matches = parse('$.output.text').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def zhipuai_chatglm_std(**kwargs):
    model = ModelAPI()
    json_output = model.invoke(model="chatglm_std", prompt=[{"role": "user", "content": kwargs.get('prompt')}],
                               top_p=kwargs.get('top_p', 0.7), temperature=kwargs.get('temperature', 0.9))
    matches = parse('$.data.choices[0].content').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def pangu_completions(**kwargs):
    chat = PanGuCompletions()
    json_output = chat.call(max_tokens=kwargs.get('max_tokens', 600), prompt="",
                            messages=[{"role": "user", "content": kwargs.get('prompt')}],
                            temperature=kwargs.get('temperature', 0.9))
    matches = parse('$.choices[0].message.content').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def tione_chat_completion(**kwargs):
    chat = TioneClient()
    content = chat.ChatCompletion(content=kwargs.get('prompt')).to_json_string()
    json_obj = json.loads(content)
    matches = parse('$.Choices[0].Message.Content').find(json_obj)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


class UnifiedSDK:
    def __init__(self):

        dwai.api_base_china = "https://dwai.shizhuang-inc.com"
        dwai.api_base_singapore = "https://openai.shizhuang-inc.com"

        self.route_map = {
            'alibaba.qwen-plus-v1': dwai_bailian_qa,
            'kingsoft.default': zhipuai_chatglm_std,
            'huawei.default': pangu_completions,
            'minimax.default': mini_qa,
            'azure.default': azure_qa,
            'tencent.default': tione_chat_completion
        }

    def call(self, model_key, **kwargs):
        func = self.route_map.get(model_key)
        if func:
            return func(**kwargs)
        else:
            raise ValueError(f"Unknown model_key: {model_key}")


dwai.api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"

if __name__ == '__main__':
    sdk = UnifiedSDK()
    print(">>>")

    # 测试微软
    resp = sdk.call('azure.default', prompt="你的参数量大概是多少？")
    print(resp)

    # 测试mini
    resp = sdk.call('minimax.default', prompt="你的参数量大概是多少？")
    print(resp)

    # 测试dwai
    resp = sdk.call('alibaba.qwen-plus-v1', prompt="你的参数量大概是多少？")
    print(resp)

    # 测试zhipuai
    resp = sdk.call('kingsoft.default', prompt="你的参数量大概是多少？", top_p=0.7, temperature=0.9)
    print(resp)

    # 测试pangu
    resp = sdk.call('huawei.default', prompt="你的参数量大概是多少？", max_tokens=600, temperature=0.9)
    print(resp)

    # 测试tione
    resp = sdk.call('tencent.default', prompt="你的参数量大概是多少？")
    print(resp)



import json
from typing import Dict, Optional, Iterator, List

from dwai.bailian.model_api.base_completions import BaseCompletions
from dwai.bailian.model_api.models import ChatQaMessage


class Completions(BaseCompletions):
    def call(self, app_id: str,
             prompt: str,
             request_id: str = None,
             session_id: str = None,
             history: List[ChatQaMessage] = None,
             top_p: float = 0.0,
             biz_params: Dict = None,
             has_thoughts: bool = False,
             stream: bool = False,
             doc_reference_type: str = None):
        resp = self.reqeust(app_id=app_id,
                            prompt=prompt,
                            request_id=request_id,
                            session_id=session_id,
                            history=history,
                            top_p=top_p,
                            biz_params=biz_params,
                            has_thoughts=has_thoughts,
                            stream=stream,
                            doc_reference_type=doc_reference_type)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data: [DONE]":
                return None
            if line.startswith(b"data: "):
                line = line[len(b"data: "):]
                return line.decode("utf-8")
            else:
                return None
        return None



import logging
from typing import Any, Optional, Dict, List

from dwai.unified.model_api.completions import UnifiedSDK
from langchain.callbacks.manager import CallbackManagerForLLMRun
from langchain.llms.base import LLM
from langchain.llms.utils import enforce_stop_tokens
from langchain.utils import get_from_dict_or_env
from pydantic.class_validators import root_validator

logger = logging.getLogger(__name__)


class DwUnifiedLLM(LLM):
    client: Any
    model_key: str = "alibaba.qwen-plus-v1"
    api_key: str = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
    api_base_china: str = "https://dwai.shizhuang-inc.com"
    api_base_singapore: str = "https://openai.shizhuang-inc.com"

    def __init__(self, model_key: str, api_key: str, api_base_china: str ="https://dwai.shizhuang-inc.com", api_base_singapore: str = "https://openai.shizhuang-inc.com", **kwargs: Any):
        super().__init__(**kwargs)
        self.model_key = model_key
        self.api_key = api_key
        self.api_base_china = api_base_china
        self.api_base_singapore = api_base_singapore

    def __call__(self, *args, **kwargs):
        return super().__call__(*args, **kwargs)

    @property
    def _llm_type(self) -> str:
        return "DwUnifiedLLM"

    @root_validator()
    def validate_environment(cls, values: Dict) -> Dict:
        if values["client"] is not None:
            return values

        values["api_key"] = get_from_dict_or_env(
            values,
            "api_key",
            "API_KEY",
        )
        values["api_base_china"] = get_from_dict_or_env(
            values,
            "api_base_china",
            "API_BASE_CHINA",
        )
        values["api_base_singapore"] = get_from_dict_or_env(
            values,
            "api_base_singapore",
            "API_BASE_SINGAPORE",
        )

        try:
            import dwai
            from dwai.unified.model_api.completions import UnifiedSDK
            dwai.api_key = values["api_key"]
            dwai.api_base_china = values["api_base_china"]
            dwai.api_base_singapore = values["api_base_singapore"]
            values["client"] = UnifiedSDK()
        except ImportError:
            raise ModuleNotFoundError(
                "Could not import UnifiedSDK python package. "
                "Please install it with `pip3.10 install --no-cache-dir  dwai==0.2.1`."
            )
        return values

    def _call(
        self,
        prompt: str,
        stop: Optional[List[str]] = None,
        run_manager: Optional[CallbackManagerForLLMRun] = None,
        **kwargs: Any,
    ) -> str:
        try:
            response = self.client.call(self.model_key, prompt=prompt)
            text = response.get("output", {}).get("text")
            if not response.get("Success"):
                raise RuntimeError(response.get("Message"))
        except Exception as e:
            raise RuntimeError(f"Error raised by broadscope service: {e}")
        if stop is not None:
            text = enforce_stop_tokens(text, stop)
        return text



from typing import Iterator, Optional

import dwai

import posixpath
import json
from dwai.zhipuai.utils.http_client import get, post, stream
from dwai.zhipuai.utils.sse_client import SSEClient


class InvokeType:
    SYNC = "invoke"
    ASYNC = "async-invoke"
    SSE = "sse-invoke"


class ModelAPI:

    @classmethod
    def invoke(cls, **kwargs):
        url = cls._build_api_url(kwargs, InvokeType.SYNC)
        return post(url, kwargs)

    @classmethod
    def async_invoke(cls, **kwargs):
        url = cls._build_api_url(kwargs, InvokeType.ASYNC)
        return post(url, kwargs)

    @classmethod
    def query_async_invoke_result(cls, task_id: str):
        url = cls._build_api_url(None, InvokeType.ASYNC, task_id)
        return get(url, )

    @classmethod
    def sse_invoke(cls, **kwargs):
        url = cls._build_api_url(kwargs, InvokeType.SSE)
        data = stream(url, kwargs)
        # return SSEClient(data)
        return (line for line in cls.parse_response_stream(data.iter_lines()))

    @staticmethod
    def _build_api_url(kwargs, *path):
        if kwargs:
            if "model" not in kwargs:
                raise Exception("model param missed")
            model = kwargs.pop("model")
        else:
            model = "-"

        url = "%s%s" % (dwai.api_base_china, "/zhipu/api/paas/v3/model-api")
        return posixpath.join(url, model, *path)

    @classmethod
    def parse_response_stream(cls, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = cls.parse_stream_line(line)
            if _line is not None:
                yield {"data": _line}

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None



# coding=utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import os
import pytest
import functools
import openai
# from devtools_testutils.sanitizers import add_header_regex_sanitizer, add_oauth_response_sanitizer


# for pytest.parametrize
ALL = ["azure", "azuread", "openai"]
AZURE = "azure"
OPENAI = "openai"
AZURE_AD = "azuread"

# Environment variable keys
ENV_AZURE_OPENAI_ENDPOINT = "AZURE_OPENAI_ENDPOINT"
ENV_AZURE_OPENAI_KEY = "AZURE_OPENAI_KEY"
ENV_SUBSCRIPTION_ID = "AZURE_SUBSCRIPTION_ID"
ENV_TENANT_ID = "AZURE_TENANT_ID"
ENV_CLIENT_ID = "AZURE_CLIENT_ID"
ENV_CLIENT_SECRET = "AZURE_CLIENT_SECRET"

ENV_AZURE_OPENAI_API_VERSION = "2023-07-01-preview"
ENV_AZURE_OPENAI_COMPLETIONS_NAME = "text-davinci-003"
ENV_AZURE_OPENAI_CHAT_COMPLETIONS_NAME = "gpt-35-turbo-0613"
ENV_AZURE_OPENAI_EMBEDDINGS_NAME = "text-embedding-ada-002"

ENV_OPENAI_KEY = "OPENAI_KEY"
ENV_OPENAI_COMPLETIONS_MODEL = "text-davinci-003"
ENV_OPENAI_CHAT_COMPLETIONS_MODEL = "gpt-3.5-turbo"
ENV_OPENAI_EMBEDDINGS_MODEL = "text-embedding-ada-002"

# Fake values
TEST_ENDPOINT = "https://test-resource.openai.azure.com/"
TEST_KEY = "0000000000000000"
TEST_ID = "00000000-0000-0000-0000-000000000000"


# @pytest.fixture(scope="session", autouse=True)
# def add_sanitizers(test_proxy, environment_variables):
#     sanitization_mapping = {
#         ENV_AZURE_OPENAI_ENDPOINT: TEST_ENDPOINT,
#         ENV_AZURE_OPENAI_KEY: TEST_KEY,
#         ENV_SUBSCRIPTION_ID: TEST_ID,
#         ENV_TENANT_ID: TEST_ID,
#         ENV_CLIENT_ID: TEST_ID,
#         ENV_CLIENT_SECRET: TEST_ID,
#         ENV_OPENAI_KEY: TEST_KEY
#     }
#     environment_variables.sanitize_batch(sanitization_mapping)
#     add_oauth_response_sanitizer()
#     add_header_regex_sanitizer(key="Set-Cookie", value="[set-cookie;]")


@pytest.fixture(scope="session")
def azure_openai_creds():
    yield {
        "completions_name": ENV_AZURE_OPENAI_COMPLETIONS_NAME,
        "chat_completions_name": ENV_AZURE_OPENAI_CHAT_COMPLETIONS_NAME,
        "embeddings_name": ENV_AZURE_OPENAI_EMBEDDINGS_NAME,
        "completions_model": ENV_OPENAI_COMPLETIONS_MODEL,
        "chat_completions_model": ENV_OPENAI_CHAT_COMPLETIONS_MODEL,
        "embeddings_model": ENV_OPENAI_EMBEDDINGS_MODEL,
    }


def configure_api_type(api_type, **kwargs):
    if api_type == "azure":
        openai.api_base = os.getenv(ENV_AZURE_OPENAI_ENDPOINT).rstrip("/")
        openai.api_type = "azure"
        openai.api_key = os.getenv(ENV_AZURE_OPENAI_KEY)
        openai.api_version = ENV_AZURE_OPENAI_API_VERSION
    elif api_type == "openai":
        openai.api_base = "https://api.openai.com/v1"
        openai.api_type = "openai"
        openai.api_key = os.getenv(ENV_OPENAI_KEY)
        openai.api_version = None


def configure_async(f):
    @functools.wraps(f)
    async def wrapper(*args, **kwargs):
        api_type = kwargs.pop("api_type")
        configure_api_type(api_type, **kwargs)
        try:
            return await f(*args, api_type=api_type, **kwargs)
        except openai.error.RateLimitError:
            pytest.skip(f"{str(f).split(' ')[1]}[{api_type}]: Skipping - Rate limit reached.")

    return wrapper


def configure(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        api_type = kwargs.pop("api_type")
        configure_api_type(api_type, **kwargs)
        try:
            return f(*args, api_type=api_type, **kwargs)
        except openai.error.RateLimitError:
            pytest.skip(f"{str(f).split(' ')[1]}[{api_type}]: Skipping - Rate limit reached.")

    return wrapper
